﻿using System;
using System.Collections.Generic;
using System.Reflection.Emit;

namespace TestProject
{
    /// <summary>
    /// 구조체 헬퍼
    /// </summary>
    public static class StructureHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 타입 딕셔너리
        /// </summary>
        private static Dictionary<Type, int> _typeDictionary = new Dictionary<Type, int>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 크기 구하기 - SizeOf(type)

        /// <summary>
        /// 크기 구하기
        /// </summary>
        /// <param name="type">타입</param>
        /// <returns>크기</returns>
        public static int SizeOf(Type type)
        {
            int size;

            if(_typeDictionary.TryGetValue(type, out size))
            {
                return size;
            }

            size = GetSize(type);

            _typeDictionary.Add(type, size);

            return size;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 크기 - GetSize(type)

        /// <summary>
        /// 크기 구하기
        /// </summary>
        /// <param name="type">타입</param>
        /// <returns>크기</returns>
        private static int GetSize(Type type)
        {
            DynamicMethod dynamicMethod = new DynamicMethod("SizeOfType", typeof(int), new Type[] { });

            ILGenerator ilGenerator = dynamicMethod.GetILGenerator();

            ilGenerator.Emit(OpCodes.Sizeof, type);
            ilGenerator.Emit(OpCodes.Ret);

            return (int)dynamicMethod.Invoke(null, null);
        }

        #endregion
    }
}